/*****************************************************************************/
/*                                                                           */
/* liburl2cat : Domain Categorization Library                                */
/*                                                                           */
/*  Copyright (C) 2020 Kuzmin Yaroslav <kuzmin.yaroslav@gmail.com>           */
/*  Copyright (C) 2022 Kuzmin Yaroslav <kuzmin.yaroslav@gmail.com>           */
/*                                                                           */
/* liburl2cat is free software: you can redistribute it and/or modify it     */
/* under the terms of the GNU General Public License as published by the     */
/* Free Software Foundation, either version 3 of the License, or             */
/* (at your option) any later version.                                       */
/*                                                                           */
/* liburl2cat is distributed in the hope that it will be useful, but         */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      */
/* See the GNU General Public License for more details.                      */
/*                                                                           */
/* You should have received a copy of the GNU General Public License along   */
/* with this program.  If not, see <http://www.gnu.org/licenses/>.           */
/*                                                                           */
/*****************************************************************************/

#ifndef URL2CAT_H
	#define URL2CAT_H

	#define CATEGORY_TYPE_NAME_STR_SIZE 256
struct url2cat_category
{
	unsigned char type;
	char type_name[CATEGORY_TYPE_NAME_STR_SIZE];
};
typedef struct url2cat_category url2cat_category_s;

	#define MAX_CHAR_LEN_DECIMAL_INTEGER(type) (3 * sizeof(type) + 2)

	#define DATABASE_NAME_STR_SIZE     1024
	#define DATABASE_USER_VERSION_SIZE 9
	#define NET_SCHEME_STR_SIZE        32
	#define NET_HOST_STR_SIZE          256
	#define NET_PORT_STR_SIZE          MAX_CHAR_LEN_DECIMAL_INTEGER(long int)
	#define NET_PATH_STR_SIZE          256
	#define NET_USER_STR_SIZE          256
	#define NET_PASSWORD_STR_SIZE      256

struct url2cat_setting
{
	size_t cache_size;

	#ifdef HAVE_DATABASE
	char db_name[DATABASE_NAME_STR_SIZE];
	char db_update_host[NET_HOST_STR_SIZE];
	char db_update_path[NET_PATH_STR_SIZE];
	char db_download_path[NET_PATH_STR_SIZE];
	char db_update_user[NET_USER_STR_SIZE];
	char db_update_password[NET_PASSWORD_STR_SIZE];
	//TODO add authorization (base64)
	char db_user_version[DATABASE_USER_VERSION_SIZE];
	#endif

	#ifdef HAVE_SERVER
	char server_host[NET_HOST_STR_SIZE];
	char server_path[NET_PATH_STR_SIZE];
	char server_user[NET_USER_STR_SIZE];
	char server_password[NET_PASSWORD_STR_SIZE];
	//TODO add authorization (base64)
	#endif

	char recat_host[NET_HOST_STR_SIZE];
	char recat_path[NET_PATH_STR_SIZE];
	char recat_user[NET_USER_STR_SIZE];
	char recat_password[NET_PASSWORD_STR_SIZE];
	//TODO add authorization (base64)
};
typedef struct url2cat_setting url2cat_setting_s;

const char* url2cat_version(void);
int url2cat_read_ini(char* file_name, url2cat_setting_s* url2cat_setting);
int url2cat_init(url2cat_setting_s* url2cat_setting);
int url2cat_deinit(void);
int url2cat_category(char* url, size_t url_size, url2cat_category_s* category, int* category_number);
int url2cat_recategory(char* url, size_t url_size, char* category_name, size_t category_name_size);
	#ifdef HAVE_DATABASE
int url2cat_update_database(url2cat_setting_s* url2cat_setting);
	#endif

enum url2cat_error {
	SUCCESS = 0x0000,
	/*******************************************************/
	CACHE_URL_NOT_FOUND = 0x0001,
	DB_URL_NOT_FOUND = 0x0002,
	SERVER_URL_NOT_FOUND = 0x0003,
	URL_END_CHECK = 0x0004,
	DB_IS_BEING_UPDATED = 0x0005,
	WARNING = 0x1000,
	/*******************************************************/
	INIT_REPEATED = 0x1001,
	INIT_DB_NAME_EMPTY = 0x1002,
	INIT_DB_UPDATE_HOST_EMPTY = 0x1003,
	INIT_DB_UPDATE_PATH_EMPTY = 0x1004,
	INIT_DB_DOWNLOAD_PATH_EMPTY = 0x1005,
	INIT_DB_UPDATE_USER_EMPTY = 0x1006,
	INIT_DB_UPDATE_PASSWORD_EMPTY = 0x1007,
	INIT_SERVER_HOST_EMPTY = 0x1008,
	INIT_SERVER_PATH_EMPTY = 0x1009,
	INIT_SERVER_USER_EMPTY = 0x100A,
	INIT_SERVER_PASSWORD_EMPTY = 0x100B,
	INIT_RECAT_HOST_EMPTY = 0x100D,
	INIT_RECAT_PATH_EMPTY = 0x100E,
	INIT_RECAT_USER_EMPTY = 0x100F,
	INIT_RECAT_PASSWORD_EMPTY = 0x1010,
	LOCALE_MALLOC_NULL = 0x1011,
	COMMON_BASE64_LEN = 0x1012,
	COMMON_BASE64_MALLOC_NULL = 0x1013,
	CACHE_SIZE_LIMITE = 0x1014,
	CACHE_BUFFER_MALLOC_NULL = 0x1015,
	CACHE_URL_NO_PATH = 0x1016,
	RECAT_HEADER_MALLOC = 0x1017,
	RECAT_CONTENT_MALLOC = 0x1018,
	RECAT_REQUEST_MALLOC = 0x1019,
	RECAT_PARSER_ANSWER = 0x101A,
	RECAT_REQUEST_WRITE = 0x101B,
	RECAT_READ_ANSWER = 0x101C,
	URL_INCORRECT = 0x101D,
	CATEGORY_ARRAY_NULL = 0x101E,
	READ_INI_NOT_OPEN_FILE = 0x101F,
	READ_INI_CALLOC_OPTIONS = 0x1020,
	READ_INI_RECALLOC_OPTIONS = 0x1021,
	READ_INI_NOT_OPTION = 0x1022,
	READ_INI_NOT_STREAM = 0x1023,
	READ_INI_MALLOC_BUFF = 0x1024,
	READ_INI_OPTIONS = 0x1025,
	/*****************************************************************************/
	CANON_BUFFER_MALLOC_NULL = 0x2001,
	CANON_BUFFER_MALLOC_REPEAT = 0x2002,
	CANON_BUFFER_REALLOC_NULL = 0x2003,
	CANON_CHAR_HEX_UP = 0x2005,
	CANON_CHAR_HEX_LOW = 0x2006,
	CANON_PATH_SLASH_DOT_DOT = 0x2007,
	/*******************************************************/
	DB_OPEN = 0x3001,
	DB_JOURNAL_MODE = 0x3002,
	DB_TEMP_STORE = 0x3003,
	DB_SYNCHRONOUS = 0x3004,
	DB_LOCKING_MODE = 0x3005,
	DB_QUERY_CATEGORY_NAME_PREPARE = 0x3006,
	DB_QUERY_CATEGORY_NAME_STEP = 0x3007,
	DB_QUERY_CATEGORY_NAME_NOT_ROW = 0x3008,
	DB_QUERY_CATEGORY_NAME_TYPE = 0x3009,
	DB_QUERY_CATEGORY_NAME = 0x300A,
	DB_QUERY_USER_VERSION_GET_PREPARE = 0x300B,
	DB_QUERY_USER_VERSION_GET_STEP = 0x300C,
	DB_USER_VERSION_GET = 0x300D,
	DB_QUERY_USER_VERSION_SET_PREPARE = 0x300E,
	DB_QUERY_USER_VERSION_SET_STEP = 0x300F,
	DB_USER_VERSION_SET = 0x3010,
	DB_REQUEST_UPDATE_HEAD_MALLOC_NULL = 0x3011,
	DB_REQUEST_UPDATE_FULL_MALLOC_NULL = 0x3012,
	DB_REQUEST_DOWNLOAD_HEAD_MALLOC_NULL = 0x3013,
	DB_REQUEST_DOWNLOAD_FULL_MALLOC_NULL = 0x3014,
	DB_PARSE_ANSWER_NOT_OK = 0x3015,
	DB_PARSE_ANSWER_NOT_CONTENT_RANGE = 0x3016,
	DB_PARSE_ANSWER_NOT_END_HEAD = 0x3017,
	DB_PARSE_ANSWER_NOT_LENGTH = 0x3018,
	DB_UPDATE_NOT_MAGIC_STR_HEAD = 0x3019,
	DB_UPDATE_NOT_USER_VERSION_HEAD = 0x301A,
	DB_DOWNLOAD_NOT_MAGIC_STR_HEAD = 0x301B,
	DB_FILE_RENAME = 0x301C,
	DB_DOWNLOAD_FILE_NOT_CREAT = 0x301D,
	DB_DOWNLOAD_FILE_NOT_FS_STAT = 0x301E,
	DB_DOWNLOAD_FILE_NOT_SPACE_DEVICE = 0x301F,
	DB_DOWNLOAD_FILE_NOT_FULL = 0x3020,
	DB_DOWNLOAD_FILE_NOT_WRITE = 0x3021,
	DB_DOWNLOAD_FILE_NOT_MMAP = 0x3022,
	DB_DOWNLOAD_FILE_READ = 0x3023,
	DB_QUERY_APPLY_PATCH_PREPARE = 0x3024,
	DB_QUERY_APPLY_PATCH_STEP = 0x3025,
	DB_QUERY_APPLY_PATCH_NOT_DONE = 0x3026,
	DB_APPLY_PATCH_NOT_MAGIC_STR = 0x3027,
	DB_APPLY_PATCH_NOT_USER_VERSION = 0x3028,
	DB_APPLY_PATCH_MISTMACH_USER_VERSION = 0x3029,
	DB_APPLY_PATCH_NOT_END_ROW = 0x302A,
	DB_UPDATE_HEADER_WRITE = 0x302B,
	DB_UPDATE_HEADER_READ = 0x302C,
	DB_UPDATE_FULL_WRITE = 0x302D,
	DB_UPDATE_FULL_READ = 0x302E,
	DB_DOWNLOAD_HEADER_WRITE = 0x302F,
	DB_DOWNLOAD_HEADER_READ = 0x3030,
	DB_DOWNLOAD_FULL_WRITE = 0x3031,
	DB_DOWNLOAD_FULL_READ = 0x3032,
	DB_QUERY_CATEGORY_MALLOC_NULL = 0x3033,
	DB_QUERY_CATEGORY_PREPARE = 0x3034,
	DB_QUERY_CATEGORY_STEP = 0x3035,
	DB_QUERY_CATEGORU_NOT_ROW = 0x3036,
	DB_URL_NO_PATH = 0x3037,
	/*******************************************************/
	SERVER_REQUEST_GET_PATH_MALLOC_NULL = 0x4001,
	SERVER_REQUEST_GET_PATH_REALLOC_NULL = 0x4002,
	SERVER_REQUEST_HOST_MALLOC_NULL = 0x4003,
	SERVER_REQUEST_BUFFER_MALLOC_NULL = 0x4004,
	SERVER_ANSWER_BUFFER_MALLOC_NULL = 0x4005,
	SERVER_ANSWER_NOT_OK = 0x4006,
	SERVER_ANSWER_NOT_CATEGORY_TYPE = 0x4007,
	SERVER_ANSWER_NOT_CATEGORY_TYPE_NAME = 0x4008,
	SERVER_ANSWER_NOT_CATEGORY_TYPE_BRACKET = 0x4009,
	SERVER_ANSWER_NOT_CATEGORY_TYPE_NAME_BRACKET = 0x400A,
	SERVER_ANSWER_NOT_CATEGORY_TYPE_INT = 0x400B,
	SERVER_CATEGORY_NOT_CONNECT = 0x400C,
	SERVER_CATEGORY_HOST_WRITE = 0x400D,
	SERVER_CATEGORY_HOST_READ = 0x400E,
	SERVER_INIT_REPEAT = 0x400F,
	/*******************************************************/
	NET_HTTPS_NOT_CONNECT = 0x5001,
	NET_SCHEME_NOT_SUPPORT = 0x5002,
	NET_HOST_PORT_MALLOC_NULL = 0x5003,
	NET_CONNECT_FAILURE = 0x5004,
	NET_WRITE_FAILURE = 0x5005,
	NET_READ_FAILURE = 0x5006,
	NET_CHECK_CONNECT_FAILURE = 0x5007,
	NET_SSL_CONF_CTX_finish = 0x5008,
	NET_SSL_BIO_get_ssl = 0x5009,
	NET_SSL_BIO_do_connect = 0x500A,
	NET_SSL_BIO_do_handshake = 0x500B,
	/*******************************************************/
};
typedef enum url2cat_error url2cat_error_e;

#endif
/*****************************************************************************/
