/*****************************************************************************/
/*                                                                           */
/* liburl2cat : Domain Categorization Library                                */
/*                                                                           */
/*  Copyright (C) 2020 Kuzmin Yaroslav <kuzmin.yaroslav@gmail.com>           */
/*                                                                           */
/* liburl2cat is free software: you can redistribute it and/or modify it     */
/* under the terms of the GNU General Public License as published by the     */
/* Free Software Foundation, either version 3 of the License, or             */
/* (at your option) any later version.                                       */
/*                                                                           */
/* liburl2cat is distributed in the hope that it will be useful, but         */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      */
/* See the GNU General Public License for more details.                      */
/*                                                                           */
/* You should have received a copy of the GNU General Public License along   */
/* with this program.  If not, see <http://www.gnu.org/licenses/>.           */
/*                                                                           */
/*****************************************************************************/

#include "common.h"

/*****************************************************************************/

char up_hex_char(uint8_t c)
{
	uint8_t t = c >> 4;
	if (t < 0xA)
		return t + 0x30;
	return t + 0x37;
}

char low_hex_char(uint8_t c)
{
	uint8_t t = c & 0x0F;
	if (t < 0xA)
		return t + 0x30;
	return t + 0x37;
}

/*****************************************************************************/
static uint8_t encoding_table[] = {
	'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
	'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
	'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/'
};

int encode_base64(uint8_t* input, size_t input_length, uint8_t** output, size_t* output_length)
{
	*output_length = 4 * ((input_length + 2) / 3);

	*output = NULL;
	if (*output_length == 0)
		return COMMON_BASE64_LEN;

	uint8_t* encoded_data = malloc(*output_length);
	if (encoded_data == NULL)
		return COMMON_BASE64_MALLOC_NULL;

	uint8_t* buf_in = input;
	uint8_t* buf_out = encoded_data;
	int i;
	uint32_t tripel;
	uint8_t a, b, c, d;
	for (i = 0; i < ((int)input_length - 2); i += 3) {
		tripel = *buf_in;
		tripel <<= 8;
		++buf_in;
		tripel += *buf_in;
		tripel <<= 8;
		++buf_in;
		tripel += *buf_in;
		++buf_in;
		a = (tripel >> (3 * 6)) & 0x3F;
		b = (tripel >> (2 * 6)) & 0x3F;
		c = (tripel >> (1 * 6)) & 0x3F;
		d = (tripel >> (0 * 6)) & 0x3F;
		*buf_out = encoding_table[a];
		++buf_out;
		*buf_out = encoding_table[b];
		++buf_out;
		*buf_out = encoding_table[c];
		++buf_out;
		*buf_out = encoding_table[d];
		++buf_out;
	}
	if (i < input_length) {
		if (i == (input_length - 1)) {
			tripel = *buf_in;
			tripel <<= 16;
			a = (tripel >> (3 * 6)) & 0x3F;
			b = (tripel >> (2 * 6)) & 0x3F;
			*buf_out = encoding_table[a];
			++buf_out;
			*buf_out = encoding_table[b];
			++buf_out;
			*buf_out = '=';
			++buf_out;
			*buf_out = '=';
		}
		else {
			tripel = *buf_in;
			tripel <<= 8;
			++buf_in;
			tripel += *buf_in;
			tripel <<= 8;
			a = (tripel >> (3 * 6)) & 0x3F;
			b = (tripel >> (2 * 6)) & 0x3F;
			c = (tripel >> (1 * 6)) & 0x3F;
			*buf_out = encoding_table[a];
			++buf_out;
			*buf_out = encoding_table[b];
			++buf_out;
			*buf_out = encoding_table[c];
			++buf_out;
			*buf_out = '=';
		}
	}
	*output = encoded_data;
	return SUCCESS;
}

/*****************************************************************************/
