/*****************************************************************************/
/*                                                                           */
/* liburl2cat : Domain Categorization Library                                */
/*                                                                           */
/*  Copyright (C) 2020 Kuzmin Yaroslav <kuzmin.yaroslav@gmail.com>           */
/*                                                                           */
/* liburl2cat is free software: you can redistribute it and/or modify it     */
/* under the terms of the GNU General Public License as published by the     */
/* Free Software Foundation, either version 3 of the License, or             */
/* (at your option) any later version.                                       */
/*                                                                           */
/* liburl2cat is distributed in the hope that it will be useful, but         */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      */
/* See the GNU General Public License for more details.                      */
/*                                                                           */
/* You should have received a copy of the GNU General Public License along   */
/* with this program.  If not, see <http://www.gnu.org/licenses/>.           */
/*                                                                           */
/*****************************************************************************/

#ifndef COMMON_H
	#define COMMON_H

/*****************************************************************************/

	#include <stdlib.h>
	#include <stdint.h>
	#include <string.h>
	#include <unistd.h>
	#include <stdio.h>
	#include <sys/types.h>
	#include <sys/stat.h>
	#include <fcntl.h>
	#include <sys/mman.h>
	#include <time.h>
	#include <sys/vfs.h>
	#include <pthread.h>

	#include <openssl/err.h>
	#include <openssl/ssl.h>
	#include <openssl/conf.h>
	#include <openssl/md5.h>

	#include <url2cat.h>

        /*****************************************************************************/
	#define NO  0
	#define YES 1

	#if HASH_FULL
		#define HASH_DIVIDER 0
	#elif HASH_HALF
		#define HASH_DIVIDER 1
	#elif HASH_QUARTER
		#define HASH_DIVIDER 2
	#else
		#error "No hash divider!!"
	#endif

	#ifdef HASH_MD5
		#define DOMAIN_HASH_SIZE (16 >> HASH_DIVIDER)
		#define PATH_HASH_SIZE   (16 >> HASH_DIVIDER)
		#define COMPUTES_HASH    MD5
	#elif HASH_SHA256
		#define DOMAIN_HASH_SIZE (32 >> HASH_DIVIDER)
		#define PATH_HASH_SIZE   (32 >> HASH_DIVIDER)
		#define COMPUTES_HASH    SHA256
	#elif HASH_SHA512
		#define DOMAIN_HASH_SIZE (64 >> HASH_DIVIDER)
		#define PATH_HASH_SIZE   (64 >> HASH_DIVIDER)
		#define COMPUTES_HASH    SHA512
	#else
		#error "Ho hash type!!"
	#endif

	#define ANSWER_BUFFER_SIZE_DEFAULT 1024

enum category_type {
	unknown_sites = 2,
	virus_propagation = 3,
	phishing = 4,
	online_ads = 5,
	drugs = 6,
	tasteless = 7,
	academic_fraud = 8,
	parked_domains = 9,
	hate_discrimination = 10,
	proxies_anonymizers = 11,
	botnets_ = 12,
	adult_sites = 13,
	alcohol_tobacco = 14,
	dating_ = 15,
	pornography_sexuality = 16,
	astrology_ = 17,
	gambling_ = 18,
	child_sexual_abuse_iwf = 19,
	torrents_p2p = 20,
	file_storage = 21,
	movies_video = 22,
	music_radio = 23,
	photo_sharing = 24,
	content_delivery_networks = 25,
	chats_messengers = 26,
	forums = 27,
	games = 28,
	social_networks = 29,
	entertainment = 30,
	german_youth_protection = 31,
	automotive = 32,
	blogs = 33,
	corporate_sites = 34,
	e_commerce = 35,
	education = 36,
	finances = 37,
	government = 38,
	health_fitness = 39,
	humor = 40,
	jobs_career = 41,
	weapons = 42,
	politics,
	_society_and_law = 43,
	news_media = 44,
	non_profit = 45,
	portals = 46,
	religious = 47,
	search_engines = 48,
	computers_internet = 49,
	sports_ = 50,
	science_technology = 51,
	travel_ = 52,
	home_family = 53,
	shopping = 54,
	arts = 55,
	webmail = 56,
	real_estate = 57,
	classifieds = 58,
	business = 59,
	kids = 60,
	justice_ministrys_federal_list = 61,
	paid_sites_of_mobile_operators = 62,
	trackers_analytics = 63,
	registry_of_banned_sites = 64,
	child_sexual_abuse_arachid = 65,
	crypto_mining = 66,
	online_libraries = 67,
	prosecutors_orders = 68,
	blackhole = 245,
	force_restricted_mode_for_youtube = 246,
	isp_black_list = 247,
	school_whitelist = 248,
	force_safe_search = 249,
	honest_nxdomain_blockpage_dos_protection = 250,
	do_not_look_into_grandbase = 251,
	global_white_list = 254,
	global_black_list = 255,
};
typedef enum category_type category_type_e;

/*****************************************************************************/

char up_hex_char(uint8_t c);
char low_hex_char(uint8_t c);

/* allocates memory for output */
int encode_base64(uint8_t* input, size_t input_length, uint8_t** output, size_t* output_length);

/*****************************************************************************/
enum scheme {
	HTTPS = 0,
	NUMBER_SCHEME = 1
};
typedef enum scheme scheme_e;

struct net_option
{
	scheme_e scheme;
	char* host;
};
typedef struct net_option net_option_s;

struct net_connect
{
	/*https support*/
	SSL_CTX* ssl_ctx;
	SSL_CONF_CTX* ssl_conf_ctx;
	BIO* bio_host;
};
typedef struct net_connect net_connect_s;
#endif

/*****************************************************************************/
