/*****************************************************************************/
/*                                                                           */
/* liburl2cat : Domain Categorization Library                                */
/*                                                                           */
/*  Copyright (C) 2020 Kuzmin Yaroslav <kuzmin.yaroslav@gmail.com>           */
/*                                                                           */
/* liburl2cat is free software: you can redistribute it and/or modify it     */
/* under the terms of the GNU General Public License as published by the     */
/* Free Software Foundation, either version 3 of the License, or             */
/* (at your option) any later version.                                       */
/*                                                                           */
/* liburl2cat is distributed in the hope that it will be useful, but         */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      */
/* See the GNU General Public License for more details.                      */
/*                                                                           */
/* You should have received a copy of the GNU General Public License along   */
/* with this program.  If not, see <http://www.gnu.org/licenses/>.           */
/*                                                                           */
/*****************************************************************************/

#include "common.h"

/*****************************************************************************/

#ifdef HAVE_DATABASE
	#define NUMBER_TYPE 256
static char* _name_category[NUMBER_TYPE] = { 0 };
#else /*! HAVE_DATABASE*/

	#ifdef LOCALE_EN
static char* _name_category[] = {
	[2] = "Unknown Sites",
	[3] = "Virus Propagation",
	[4] = "Phishing",
	[5] = "Online Ads",
	[6] = "Drugs",
	[7] = "Tasteless",
	[8] = "Academic Fraud",
	[9] = "Parked Domains",
	[10] = "Hate & Discrimination",
	[11] = "Proxies & Anonymizers",
	[12] = "Botnets",
	[13] = "Adult Sites",
	[14] = "Alcohol & Tobacco",
	[15] = "Dating",
	[16] = "Pornography & Sexuality",
	[17] = "Astrology",
	[18] = "Gambling",
	[19] = "Child Sexual Abuse (IWF)",
	[20] = "Torrents & P2P",
	[21] = "File Storage",
	[22] = "Movies & Video",
	[23] = "Music & Radio",
	[24] = "Photo Sharing",
	[25] = "Content Delivery Networks",
	[26] = "Chats & Messengers",
	[27] = "Forums",
	[28] = "Games",
	[29] = "Social Networks",
	[30] = "Entertainment",
	[31] = "German Youth Protection",
	[32] = "Automotive",
	[33] = "Blogs",
	[34] = "Corporate Sites",
	[35] = "E-commerce",
	[36] = "Education",
	[37] = "Finances",
	[38] = "Government",
	[39] = "Health & Fitness",
	[40] = "Humor",
	[41] = "Jobs & Career",
	[42] = "Weapons",
	[43] = "Politics, Society and Law",
	[44] = "News & Media",
	[45] = "Non-profit",
	[46] = "Portals",
	[47] = "Religious",
	[48] = "Search Engines",
	[49] = "Computers & Internet",
	[50] = "Sports",
	[51] = "Science & Technology",
	[52] = "Travel",
	[53] = "Home & Family",
	[54] = "Shopping",
	[55] = "Arts",
	[56] = "Webmail",
	[57] = "Real Estate",
	[58] = "Classifieds",
	[59] = "Business",
	[60] = "Kids",
	[61] = "Justice Ministry's federal list",
	[62] = "Paid sites of mobile operators",
	[63] = "Trackers & Analytics",
	[64] = "Registry of banned sites",
	[65] = "Child Sexual Abuse (Arachnid)",
	[66] = "Crypto Mining",
	[67] = "Online Libraries",
	[68] = "Prosecutors orders",
	[245] = "Blackhole",
	[246] = "Force Restricted Mode for Youtube",
	[247] = "ISP Black List",
	[248] = "School Whitelist",
	[249] = "Force Safe Search",
	[250] = "Honest NXDOMAIN (blockpage DoS protection)",
	[251] = "Do Not Look Into Grandbase",
	[254] = "Global White List",
	[255] = "Global Black List",
};
	#else
		#error "Locale not set !!"
	#endif /* LOCALE_ */

#endif /* !HAVE_DATABASE */

#ifdef HAVE_DATABASE
int add_category_name(uint8_t type, const unsigned char* name, size_t len)
{
	char* name_dest = _name_category[type];
	if (name_dest != NULL)
		free(name_dest);
	name_dest = calloc(sizeof(unsigned char), len + 1);
	if (name_dest == NULL)
		return LOCALE_MALLOC_NULL;
	memmove(name_dest, name, len);
	_name_category[type] = name_dest;
	return SUCCESS;
}

void free_category_names(void)
{
	char** current = _name_category;
	for (int i = 0; i < NUMBER_TYPE; ++i, ++current) {
		char* name = *current;
		if (name != NULL) {
			free(name);
			name = NULL;
		}
	}
}
#endif

const char* get_category_name(uint8_t type)
{
	return _name_category[type];
}

/******************************************************************************/
