/*****************************************************************************/
/*                                                                           */
/* liburl2cat : Domain Categorization Library                                */
/*                                                                           */
/*  Copyright (C) 2020 Kuzmin Yaroslav <kuzmin.yaroslav@gmail.com>           */
/*                                                                           */
/* liburl2cat is free software: you can redistribute it and/or modify it     */
/* under the terms of the GNU General Public License as published by the     */
/* Free Software Foundation, either version 3 of the License, or             */
/* (at your option) any later version.                                       */
/*                                                                           */
/* liburl2cat is distributed in the hope that it will be useful, but         */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      */
/* See the GNU General Public License for more details.                      */
/*                                                                           */
/* You should have received a copy of the GNU General Public License along   */
/* with this program.  If not, see <http://www.gnu.org/licenses/>.           */
/*                                                                           */
/*****************************************************************************/

#include "common.h"
#include "net.h"
#include "locale.h"

/*****************************************************************************/
static char* req_header_buf = NULL;
static size_t req_header_buf_len = 0;
static size_t req_header_buf_fill_len = 0;
static char* req_header_content_len = NULL;
//TODO merge header http request
static int request_header_init(char* host_str, char* port_str, char* path_str, char* user_str, char* password_str)
{
	size_t host_str_len = strlen(host_str);
	size_t port_str_len = strlen(port_str);
	size_t path_str_len = strlen(path_str);
	size_t user_str_len = strlen(user_str);
	size_t password_str_len = strlen(password_str);
	char credentials_str[(user_str_len + password_str_len + 1 + 1)];
	*credentials_str = 0;
	strcat(credentials_str, user_str);
	strcat(credentials_str, ":");
	strcat(credentials_str, password_str);
	uint8_t* authorization;
	size_t authorization_len;
	int rc = encode_base64((uint8_t*)credentials_str, (user_str_len + password_str_len + 1), &authorization, &authorization_len);
	if (rc != SUCCESS)
		return rc;

#define REQUEST_HEADER_CONTENT_LENGHT_LEN 9
#define REQUEST_HEADER_MAIN_LEN           (5 + 17 + 22 + 2 + 1 + 3 + 23 + 2 + 13 + 36 + 16)
	req_header_buf_len = REQUEST_HEADER_MAIN_LEN + path_str_len + host_str_len + authorization_len + REQUEST_HEADER_CONTENT_LENGHT_LEN;
	if (port_str_len != 0)
		req_header_buf_len += (port_str_len + 1);
	req_header_buf = malloc(req_header_buf_len + 1);
	if (req_header_buf == NULL)
		return RECAT_HEADER_MALLOC;

	char* buf = req_header_buf;
	memmove(buf, "POST ", 5);
	buf += 5;
	memmove(buf, path_str, path_str_len);
	buf += path_str_len;
	memmove(buf, " HTTP/1.1\r\n"
	             "Host: ",
	        17);
	buf += 17;
	memmove(buf, host_str, host_str_len);
	buf += host_str_len;
	if (port_str_len != 0) {
		*buf = ':';
		++buf;
		memmove(buf, port_str, port_str_len);
		buf += port_str_len;
	}
	memmove(buf, "\r\n"
	             "User-Agent: url2cat-",
	        22);
	buf += 22;
	char version_major[3] = { 0 };
	snprintf(version_major, 3, "%02d", PROJECT_VERSION_MAJOR);
	memmove(buf, version_major, 2);
	buf += 2;
	*buf = '.';
	++buf;
	char version_minor[4] = { 0 };
	snprintf(version_minor, 4, "%03d", PROJECT_VERSION_MINOR);
	memmove(buf, version_minor, 3);
	buf += 3;
	memmove(buf, "\r\n"
	             "Authorization: Basic ",
	        23);
	buf += 23;
	memmove(buf, authorization, authorization_len);
	buf += authorization_len;
	memmove(buf, "\r\n"
	             "Accept: */*\r\n"
	             "Content-Type: application/json-rpc\r\n"
	             "Content-Length: ",
	        67);
	buf += 67;
	req_header_content_len = buf;
	//memmove(buf, "00000\r\n\r\n", REQUEST_HEADER_CONTENT_LENGHT_LEN);
	free(authorization);
	return SUCCESS;
}

static int request_header_deinit(void)
{
	if (req_header_buf != NULL) {
		free(req_header_buf);
		req_header_buf = NULL;
	}
	return SUCCESS;
}

static char* req_content_buf = NULL;
static size_t req_content_buf_fill_len = 0;
static size_t req_content_buf_len = 0;

#define METOD_NAME               "categorizeUrl"
#define METOD_NAME_SIZE          13
#define REQUEST_CONTENT_ID_LEN   5
#define REQUEST_CONTENT_MAIN_LEN (19 + 11 + METOD_NAME_SIZE + 3 + 12 + 4 + 10)
/* {"id":0, "jsonrpc":"2.0", "method":"categorizeRequest", "params":["url", "cateory"]} */

static int request_content_init(size_t len)
{
	if (req_content_buf != NULL)
		free(req_content_buf);
	req_content_buf = malloc((REQUEST_CONTENT_MAIN_LEN + REQUEST_CONTENT_ID_LEN + len + 1));
	if (req_content_buf == NULL)
		return RECAT_CONTENT_MALLOC;
	req_content_buf_len = len;

	return SUCCESS;
}

static int request_content_deinit(void)
{
	if (req_content_buf != NULL) {
		free(req_content_buf);
		req_content_buf = NULL;
	}
	return SUCCESS;
}

#define MAX_ID 100000
static int content_id = 0;

static int request_content_fill(char* url, size_t url_size, char* category_name, size_t category_name_size)
{
	int rc;
	if (req_content_buf_len < (url_size + category_name_size)) {
		rc = request_content_init((url_size + category_name_size));
		if (rc != SUCCESS)
			return rc;
	}
	req_content_buf_fill_len = 0;
	char* buf = req_content_buf;

	memmove(buf, "{\"jsonrpc\": \"2.0\", "              // 19
	             "\"method\": \"" METOD_NAME "\", "     // 11 + 3
	             "\"params\": [\"",                     // 12
	        (19 + 11 + METOD_NAME_SIZE + 3 + 12));
	buf += (19 + 11 + METOD_NAME_SIZE + 3 + 12);
	req_content_buf_fill_len += (19 + 11 + METOD_NAME_SIZE + 3 + 12);
	memmove(buf, url, url_size);
	buf += url_size;
	req_content_buf_fill_len += url_size;
	memmove(buf, "\", \"", 4);
	buf += 4;
	req_content_buf_fill_len += 4;
	memmove(buf, category_name, category_name_size);
	buf += category_name_size;
	req_content_buf_fill_len += category_name_size;
	memmove(buf, "\"], \"id\": ", 10);
	buf += 10;
	req_content_buf_fill_len += 10;
	rc = snprintf(buf, REQUEST_CONTENT_ID_LEN, "%d}", content_id);
	++content_id;
	if (content_id >= MAX_ID)
		content_id = 0;
	req_content_buf_fill_len += rc;
	*(req_content_buf + req_content_buf_fill_len) = 0;
	return SUCCESS;
}

static int request_header_fill(void)
{
	req_header_buf_fill_len = req_header_buf_len - REQUEST_HEADER_CONTENT_LENGHT_LEN;
	int len = snprintf(req_header_content_len, REQUEST_HEADER_CONTENT_LENGHT_LEN, "%ld\r\n\r\n", req_content_buf_fill_len);
	req_header_buf_fill_len += len;
	return SUCCESS;
}

static int parse_answer(char* buffer, size_t len)
{
	char* rc = strstr(buffer, "error");
	if (rc != NULL)
		return RECAT_PARSER_ANSWER;

	return SUCCESS;
}

static char* req_buf = NULL;
static size_t req_buf_len = 0;

static int request_buf_init(size_t len)
{
	if (req_buf != NULL)
		free(req_buf);
	req_buf = malloc(len);
	if (req_buf == NULL)
		return RECAT_REQUEST_MALLOC;
	return SUCCESS;
}

static int request_buf_deinit(void)
{
	if (req_buf != NULL) {
		free(req_buf);
		req_buf = NULL;
	}
	return SUCCESS;
}

static net_option_s net_option = { 0 };

int send_recategory(char* url, size_t url_size, char* category_name, size_t category_name_size)
{
	net_connect_s net_connect;
	int rc = connect_net(&net_option, &net_connect);
	if (rc != SUCCESS)
		return rc;

	rc = request_content_fill(url, url_size, category_name, category_name_size);
	if (rc != SUCCESS)
		goto send_recategory_exit;

	rc = request_header_fill();
	if (rc != SUCCESS)
		goto send_recategory_exit;

	size_t buf_fill_len = req_header_buf_fill_len + req_content_buf_fill_len;
	if (buf_fill_len > req_buf_len) {
		rc = request_buf_init(buf_fill_len);
		if (rc != SUCCESS)
			goto send_recategory_exit;
	}

	char* buf = req_buf;
	memmove(buf, req_header_buf, req_header_buf_fill_len);
	buf += req_header_buf_fill_len;
	memmove(buf, req_content_buf, req_content_buf_fill_len);

	rc = send_net(&net_connect, req_buf, buf_fill_len);
	if (rc != buf_fill_len) {
		rc = RECAT_REQUEST_WRITE;
		goto send_recategory_exit;
	}

	char answer_buffer[ANSWER_BUFFER_SIZE_DEFAULT];
	rc = recv_net(&net_connect, answer_buffer, (ANSWER_BUFFER_SIZE_DEFAULT - 1));
	if (rc <= 0) {
		rc = RECAT_READ_ANSWER;
		goto send_recategory_exit;
	}
	*(answer_buffer + rc) = 0;

	rc = parse_answer(answer_buffer, rc);

send_recategory_exit:
	disconnect_net(&net_connect);
	return rc;
}

int init_recategory(char* scheme_str, char* host_str, char* port_str, char* path_str, char* user_str, char* password_str)
{
	int rc;
	rc = request_header_init(host_str, port_str, path_str, user_str, password_str);
	if (rc != SUCCESS)
		return rc;

	rc = init_net(&net_option, scheme_str, host_str, port_str);
	return SUCCESS;
}

int deinit_recategory(void)
{
	request_content_deinit();
	request_header_deinit();
	request_buf_deinit();
	deinit_net(&net_option);
	return SUCCESS;
}

/*****************************************************************************/
